String getContentType(String filename){
  if(server.hasArg("download")) return "application/octet-stream";
  else if(filename.endsWith(".htm")) return "text/html";
  else if(filename.endsWith(".html")) return "text/html";
  else if(filename.endsWith(".css")) return "text/css";
  else if(filename.endsWith(".js")) return "application/javascript";
  else if(filename.endsWith(".png")) return "image/png";
  else if(filename.endsWith(".gif")) return "image/gif";
  else if(filename.endsWith(".jpg")) return "image/jpeg";
  else if(filename.endsWith(".ico")) return "image/x-icon";
  else if(filename.endsWith(".xml")) return "text/xml";
  else if(filename.endsWith(".pdf")) return "application/x-pdf";
  else if(filename.endsWith(".zip")) return "application/x-zip";
  else if(filename.endsWith(".gz")) return "application/x-gzip";
  return "text/plain";
}

bool handleFileRead(String path) { // send the right file to the client (if it exists)
  Serial.println("handleFileRead: " + path);
  if (path.endsWith("/")) path += "index.html";          // If a folder is requested, send the index file
  String contentType = getContentType(path);             // Get the MIME type
  String pathWithGz = path + ".gz";
  if (SPIFFS.exists(pathWithGz) || SPIFFS.exists(path)) { // If the file exists, either as a compressed archive, or normal
    if (SPIFFS.exists(pathWithGz))                         // If there's a compressed version available
      path += ".gz";                                         // Use the compressed verion
    File file = SPIFFS.open(path, "r");                    // Open the file
    size_t sent = server.streamFile(file, contentType);    // Send it to the client
    file.close();                                          // Close the file again
    Serial.println(String("\tSent file: ") + path);
    return true;
  }
  Serial.println(String("\tFile Not Found: ") + path);   // If the file doesn't exist, return false
  return false;
}



void handleFileUpload(){ // upload a new file to the SPIFFS
  HTTPUpload& upload = server.upload();
  String filename;
  if(upload.status == UPLOAD_FILE_START){
    if(upload.filename.endsWith(".tft")&&!filesize){    
      Serial.print("Initalising Uploadfile, Filename: "); Serial.println(upload.filename);
      TelnetMsg("Initalising Uploadfile, Filename: " + String(upload.filename));
    }
    else if(upload.filename.endsWith(".tft")&&filesize){    
      NextionSend("sleep=0");  
      String status = "";
      Serial.print("Uploading File to Nextion, Filename: "); Serial.println(upload.filename);
      TelnetMsg("Uploading File to Nextion, Filename: " + String(upload.filename));      
      ESP.wdtDisable();      
      bool result = nex_download.uploadBegin(status, filesize);
      ESP.wdtEnable(WDTO_8S);      
      if(!result) {
        server.sendHeader("Location","/failure.html?reason=" + status);      // Redirect the client to the success page
        server.send(303);
        filesize = 0;
      }      
    }
    else {
      filename = "/" + upload.filename;      
      Serial.print("handleFileUpload Name: "); Serial.println(filename);
      TelnetMsg("handleFileUpload Name: " + filename);
      fsUploadFile = SPIFFS.open(filename, "w");            // Open the file for writing in SPIFFS (create if it doesn't exist)
      filename = String();
    }
   
  } else if(upload.status == UPLOAD_FILE_WRITE){
    if(upload.filename.endsWith(".tft")&&!filesize){    
      Serial.print(".");
      TelnetMsg(".");
    }
    else if(upload.filename.endsWith(".tft")&&filesize){    
      if (UploadSize <= 0){
        UploadSize = upload.currentSize;
        TelnetMsg(String(UploadSize));
        for(uint16_t j = 0; j < UploadSize; j++)
        {        
          UploadBuf[j] = upload.buf[j];
        }        
        if (UploadSize < 2048){
          ESP.wdtDisable();                
          bool result = nex_download.uploadData(UploadBuf, UploadSize);        
          ESP.wdtEnable(WDTO_8S);          
        }
      }
      else {
        for(uint16_t j = 0; j < upload.currentSize ; j++)
        {             
          UploadBuf[j + UploadSize] = upload.buf[j];
        }        
        UploadSize = UploadSize + upload.currentSize;    
        ESP.wdtDisable();      
        bool result = nex_download.uploadData(UploadBuf, UploadSize);        
        ESP.wdtEnable(WDTO_8S);                   
        UploadSize = 0;
        Serial.print(".");
        TelnetMsg(".");
        if(!result) {
          server.sendHeader("Location","/failure.html?reason=");      // Redirect the client to the success page
          server.send(303);
          filesize = 0;
        }     
      } 
      
    }
    else{
      TelnetMsg("handleFileUpload currentSize: " + upload.currentSize);
      if(fsUploadFile)
        fsUploadFile.write(upload.buf, upload.currentSize); // Write the received bytes to the file
    }

  } else if(upload.status == UPLOAD_FILE_END){
      // If the file was successfully created
      //Wenn .tft File dann UpdateNextion
    if(upload.filename.endsWith(".tft")&&!filesize){    
      filesize = upload.totalSize;      
      Serial.println();
      Serial.print("Filesize: "); Serial.println(filesize);
      TelnetMsg("Filesize: " + String(filesize));
      server.send(200, "text/plain", "File initalized, please start Upload again");      
    }
    else if(upload.filename.endsWith(".tft")&&filesize){    
      server.sendHeader("Location","/success.html");      // Redirect the client to the success page
      server.send(303);
      filesize = 0;
      TelnetMsg("Upload Success, Restarting ESP");        
      Serial.println("Upload Success, Restarting ESP");        
      ESP.restart();
    }
    else { 
      if(fsUploadFile) {
        fsUploadFile.close();                               // Close the file again
        Serial.print("handleFileUpload Size: "); Serial.println(upload.totalSize);
        TelnetMsg("handleFileUpload Size: " + upload.totalSize);
        TelnetMsg("Restarting ESP");        
        Serial.println("Restarting ESP");        
        ESP.restart();
      } else {
        server.send(500, "text/plain", "500: couldn't create file");
        TelnetMsg("500: couldn't create file");
      }
        
    }
  }
}
