

#include <ESP8266WiFi.h>
#include <WiFiClient.h>

#include <ESP8266WebServer.h>
#include <ESP8266mDNS.h>
#include <ESP8266HTTPClient.h>

#include <FS.h>
#include <SoftwareSerial.h>
#include "ESPNexUploadStream.h"
#include <ArduinoOTA.h>

#include <WiFiUdp.h>
SoftwareSerial softSerial(5, 4); /* For Wemos D1 mini RX:D1/5, TX:D2/4 */
ESPNexUploadStream nex_download(57600, &softSerial);

String req;
String ssid = "";
String password = "";
String host = "";
String url = "";
String timetosleep = "";
String wakeontouch = "";
String nexwifistatus = "";
size_t filesize=0;
uint8_t UploadBuf[4096];
uint16_t UploadSize = 0;

bool WifiConnected = false;

int i = 0;
int j = 0;


bool ConnectionEstablished; // Flag for successfully handled connection
char buffer[200];  

#define MAX_TELNET_CLIENTS 2
 
WiFiServer TelnetServer(22);
WiFiClient TelnetClient[MAX_TELNET_CLIENTS];

#define DEBUG_ESP_OTA;

HTTPClient http;

char inChar;

ESP8266WebServer server(80);

//holds the current upload
File fsUploadFile;

#define HTTP_UPLOAD_BUFLEN 4096

void setup(void){


  Serial.begin(115200);
  Serial.print("\n");
  
  Serial.setDebugOutput(true);
  
  SPIFFS.begin();  
  
  loadConfig();

  NextionSend("sleep=0");        
  NextionSend(nexwifistatus + "0");

  //WIFI INIT
  Serial.print("Connecting to ");
  Serial.println(ssid);
  if (String(WiFi.SSID()) != ssid) {
    WiFi.begin(ssid.c_str(), password.c_str());
  }
  
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println("");
  Serial.print("Connected! IP address: ");
  Serial.println(WiFi.localIP());

  MDNS.begin(host.c_str());
//  Serial.print("http://");
//  Serial.print(host);
//  Serial.println(".local");
  
  
  server.on ( "/set_nextion", set_nextion);
  server.onNotFound ( handleNotFound );
  server.begin();

  server.on ( "/restart", restart );
  server.onNotFound ( handleNotFound );
  server.begin();
  
  
  //SERVER INIT
  server.on("/", HTTP_POST,                       // if the client posts to the upload page
    [](){ server.send(200); },                          // Send status 200 (OK) to tell the client we are ready to receive
    handleFileUpload                                    // Receive and save the file
  );

  //called when the url is not defined here
  //use it to load content from SPIFFS
  server.onNotFound([](){
    if (server.uri()=="/cl_conf.txt"){
      server.send(404, "text/plain", "AccessNotAllowed");
    }
    if(!handleFileRead(server.uri())){
      server.send(404, "text/plain", "FileNotFound");
    }
  });
  
  
  Serial.println("Starting Http server");
  server.begin();

  //Serial.println("Starting Telnet server");
  TelnetServer.begin();
  TelnetServer.setNoDelay(true);

  // Start OTA server.
  Serial.println("Starting OTA server");  
  ArduinoOTA.setHostname((const char *)host.c_str());
  ArduinoOTA.setPort(8266);
  ArduinoOTA.begin();

  NextionSend("thsp=" + timetosleep);
  NextionSend("thup=" + wakeontouch);

}
 
void loop(void){
  String payload;
  String answer_id;
  server.handleClient();

  if (nexwifistatus != ""){
    if ((WiFi.status() == WL_CONNECTED) & !WifiConnected){
      NextionSend("sleep=0");        
      NextionSend(nexwifistatus + "1");
      WifiConnected = true;
     } else if ((WiFi.status() != WL_CONNECTED) & WifiConnected){
      NextionSend("sleep=0");        
      NextionSend(nexwifistatus + "0");
      WifiConnected = false;
     }
  }

  if (softSerial.available()){
    req = "";    
    while (softSerial.available() > 0)  {
      inChar = softSerial.read();
      req += inChar;
      delay(1);
    }
    req.remove(0,1); //Erstes Zeichen löschen
    req.remove(req.length() - 3);    //letzten 3 Zeichen löschen
    if (req.startsWith("SET ")){
      req.remove(0,4); //Erste 4 Zeichen löschen
      payload = http_get(req);
    }
    else if (req.startsWith("GET ")){
      req.remove(0,4); //Erste 4 Zeichen löschen
      answer_id = req.substring(req.indexOf(";")+1);
      req = req.substring(0,req.indexOf(";"));
      payload = http_get(req);
      answer_id = answer_id + getParam(payload,"value\": \"","\"") ;
      Serial.println("Nextion command: " + answer_id);
      TelnetMsg("Nextion command: " + answer_id);
      nextion_set(answer_id);     
    }
    
  }
  ArduinoOTA.handle();  // Handle OTA
  Telnet();  // Handle telnet connections
  
}
